""""Console API for atomic6ghg"""
# __main__.py


def main() -> None:
    """Reflect a description of the current package"""
    print("Package under construction.")


if __name__ == "__main__":
    main()