""" init """
from .heat_content_factors import HeatContentFactors
from .stationary_combustion_emission_factors import StationaryCombustionEmissionFactors
from .refrigerants_gwp_factors import RefrigerantsGwpFactors
from .refrigeration_and_ac_equipment_emission_factors import RefrigerationAndAcEquipmentEmissionFactors
from .unit_conversions_factors import UnitConversionsFactors
from .molecular_weights_factors import MolecularWeightsFactors
from .electricity_emission_factors import ElectricityFactors
from .refrigerants_gwp_factors import RefrigerantsGwpFactors
from .fire_suppression_leak_rates_factors import FireSuppressionFactors
from .business_travel_factors import BusinessTravelFactors
from .product_transport_emission_factors import ProductTransportEmissionFactors
from .waste_emission_factors import WasteEmissionFactors
from .mobile_combustion_ch4_and_n2o_emission_factors import MobileCombustionCh4AndN2oEmissionFactors
from .mobile_combustion_co2_emission_factors import MobileCombustionCo2EmissionFactors

heat_content_factors = HeatContentFactors()
stationary_combustion_emission_factors = StationaryCombustionEmissionFactors()
refrigerants_gwp_factors = RefrigerantsGwpFactors()
unit_conversions_factors = UnitConversionsFactors()
molecular_weights_factors = MolecularWeightsFactors()
electricity_emission_factors = ElectricityFactors()
refrigerants_gwp_factors = RefrigerantsGwpFactors()
refrigeration_and_ac_equipment_emission_factors = RefrigerationAndAcEquipmentEmissionFactors()
fire_suppression_leak_rates_factors = FireSuppressionFactors()
business_travel_factors = BusinessTravelFactors()
product_transport_emission_factors = ProductTransportEmissionFactors()
waste_emission_factors = WasteEmissionFactors()
mobile_combustion_ch4_and_n2o_emission_factors = MobileCombustionCh4AndN2oEmissionFactors()
mobile_combustion_co2_emission_factors = MobileCombustionCo2EmissionFactors()
